/********************************************************************************
** Form generated from reading UI file 'eventview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EVENTVIEW_H
#define UI_EVENTVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_EventViewWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLineEdit *accessibleFilterEdit;
    QLineEdit *roleFilterEdit;
    QComboBox *filterComboBox;
    QPushButton *clearButton;
    QTreeView *eventListView;

    void setupUi(QWidget *EventViewWidget)
    {
        if (EventViewWidget->objectName().isEmpty())
            EventViewWidget->setObjectName("EventViewWidget");
        EventViewWidget->resize(381, 301);
        verticalLayout = new QVBoxLayout(EventViewWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        accessibleFilterEdit = new QLineEdit(EventViewWidget);
        accessibleFilterEdit->setObjectName("accessibleFilterEdit");

        horizontalLayout->addWidget(accessibleFilterEdit);

        roleFilterEdit = new QLineEdit(EventViewWidget);
        roleFilterEdit->setObjectName("roleFilterEdit");

        horizontalLayout->addWidget(roleFilterEdit);

        filterComboBox = new QComboBox(EventViewWidget);
        filterComboBox->setObjectName("filterComboBox");
        filterComboBox->setMaxVisibleItems(15);

        horizontalLayout->addWidget(filterComboBox);

        clearButton = new QPushButton(EventViewWidget);
        clearButton->setObjectName("clearButton");

        horizontalLayout->addWidget(clearButton);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 1);
        horizontalLayout->setStretch(2, 1);

        verticalLayout->addLayout(horizontalLayout);

        eventListView = new QTreeView(EventViewWidget);
        eventListView->setObjectName("eventListView");
        eventListView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        eventListView->setProperty("showDropIndicator", QVariant(false));
        eventListView->setRootIsDecorated(true);
        eventListView->setItemsExpandable(true);
        eventListView->setAllColumnsShowFocus(true);
        eventListView->setHeaderHidden(false);
        eventListView->setExpandsOnDoubleClick(false);
        eventListView->header()->setVisible(true);
        eventListView->header()->setDefaultSectionSize(150);

        verticalLayout->addWidget(eventListView);


        retranslateUi(EventViewWidget);

        filterComboBox->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(EventViewWidget);
    } // setupUi

    void retranslateUi(QWidget *EventViewWidget)
    {
        EventViewWidget->setWindowTitle(tr2i18n("Form", nullptr));
        accessibleFilterEdit->setPlaceholderText(tr2i18n("Accessible Filter", nullptr));
#if QT_CONFIG(accessibility)
        roleFilterEdit->setAccessibleName(tr2i18n("Role Filter", nullptr));
#endif // QT_CONFIG(accessibility)
        roleFilterEdit->setInputMask(QString());
        roleFilterEdit->setPlaceholderText(tr2i18n("Role Filter", nullptr));
#if QT_CONFIG(accessibility)
        filterComboBox->setAccessibleName(tr2i18n("Event Filter", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        clearButton->setAccessibleName(tr2i18n("Clear Events", nullptr));
#endif // QT_CONFIG(accessibility)
        clearButton->setText(tr2i18n("&Clear", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EventViewWidget: public Ui_EventViewWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EVENTVIEW_H

